@@ -59,9 +59,6 @@
 static double internal_dpi_x = RSVG_DEFAULT_DPI_X;
 static double internal_dpi_y = RSVG_DEFAULT_DPI_Y;
 
-void
-rsvg_drawing_ctx_init(RsvgDrawingCtx * handle);
-
 static void
 rsvg_ctx_free_helper (gpointer key, gpointer value, gpointer user_data)
 {
@@ -604,6 +601,9 @@ static void
 rsvg_characters (void *data, const xmlChar *ch, int len)
 {
 	RsvgHandle *ctx = (RsvgHandle *)data;
+	char * utf8 = NULL;
+	RsvgNodeChars * self;
+	GString * string;
 	
 	if (ctx->handler && ctx->handler->characters != NULL)
 		{
@@ -611,10 +611,6 @@ rsvg_characters (void *data, const xmlChar *ch, int len)
 			return;
 		}
 
-	char * utf8 = NULL;
-	RsvgNodeChars * self;
-	GString * string;
-
 	if (!ch || !len)
 		return;
 
@@ -757,6 +753,8 @@ rsvg_get_base_uri_from_filename(const gchar * file_name)
 void rsvg_handle_set_base_uri (RsvgHandle *handle,
 							   const char *base_uri)
 {
+	g_return_if_fail(handle);
+
 	if (base_uri) {
 		if (handle->base_uri)
 			g_free (handle->base_uri);
@@ -774,6 +772,7 @@ void rsvg_handle_set_base_uri (RsvgHandle *handle,
  */
 G_CONST_RETURN char *rsvg_handle_get_base_uri (RsvgHandle *handle)
 {
+	g_return_val_if_fail(handle, NULL);
 	return handle->base_uri;
 }
 
@@ -918,6 +917,8 @@ rsvg_handle_free_impl (RsvgHandle *handle)
  */
 G_CONST_RETURN char *rsvg_handle_get_metadata (RsvgHandle *handle)
 {
+	g_return_val_if_fail(handle, NULL);
+
 	if (handle->metadata)
 		return handle->metadata->str;
 	else
@@ -937,6 +938,8 @@ G_CONST_RETURN char *rsvg_handle_get_metadata (RsvgHandle *handle)
  */
 G_CONST_RETURN char *rsvg_handle_get_title (RsvgHandle *handle)
 {
+	g_return_val_if_fail(handle, NULL);
+
 	if (handle->title)
 		return handle->title->str;
 	else
@@ -956,6 +959,8 @@ G_CONST_RETURN char *rsvg_handle_get_title (RsvgHandle *handle)
  */
 G_CONST_RETURN char *rsvg_handle_get_desc (RsvgHandle *handle)
 {
+	g_return_val_if_fail(handle, NULL);
+
 	if (handle->desc)
 		return handle->desc->str;
 	else
@@ -1115,8 +1120,6 @@ _rsvg_find_bbox (RsvgHandle *handle)
 
 	rsvg_state_push(ctx);
 	_rsvg_affine_identity(rsvg_state_current(ctx)->affine);
-
-	rsvg_state_push(ctx);
 	_rsvg_node_draw_children ((RsvgNode *)handle->treebase, ctx, 0);
 	rsvg_state_pop(ctx);
 
@@ -1132,6 +1135,10 @@ rsvg_handle_get_dimensions(RsvgHandle * handle, RsvgDimensionData * output)
 	RsvgNodeSvg * sself;
 	RsvgBbox bbox;
 
+	g_return_if_fail(output);
+	memset(output, 0, sizeof(RsvgDimensionData));
+	g_return_if_fail(handle);
+
 	sself = (RsvgNodeSvg *)handle->treebase;	
 	if(!sself) {
 		memset(output, 0, sizeof(RsvgDimensionData));
@@ -1300,6 +1307,8 @@ rsvg_handle_write (RsvgHandle    *handle,
 				   gsize          count,
 				   GError       **error)
 {
+	g_return_val_if_fail(handle, FALSE);
+
 	if (handle->first_write) {
 		handle->first_write = FALSE;
 
@@ -1341,6 +1350,8 @@ gboolean
 rsvg_handle_close (RsvgHandle  *handle,
 				   GError     **error)
 {
+	g_return_val_if_fail(handle, FALSE);
+
 #if HAVE_SVGZ
 	if (handle->is_gzipped) {
 		GsfInput * gzip;
@@ -1396,6 +1407,8 @@ rsvg_handle_close (RsvgHandle  *handle,
 void
 rsvg_handle_free (RsvgHandle *handle)
 {
+	g_return_if_fail(handle);
+
 #if HAVE_SVGZ
 	if (handle->is_gzipped)
 		g_object_unref (G_OBJECT (handle->gzipped_data));
@@ -1621,5 +1634,5 @@ void _rsvg_pop_view_box(RsvgDrawingCtx *ctx)
 {
 	ctx->vb = *((RsvgViewBox *)ctx->vb_stack->data);
 	g_free(ctx->vb_stack->data);
-	ctx->vb_stack = g_slist_remove_link(ctx->vb_stack, ctx->vb_stack);
+	ctx->vb_stack = g_slist_delete_link(ctx->vb_stack, ctx->vb_stack);
 }
