@@ -49,20 +49,8 @@ rsvg_pixmap_destroy (gchar *pixels, gpointer data)
   g_free (pixels);
 }
 
-/**
- * rsvg_handle_get_pixbuf:
- * @handle: An #RsvgHandle
- *
- * Returns the pixbuf loaded by #handle.  The pixbuf returned will be reffed, so
- * the caller of this function must assume that ref.  If insufficient data has
- * been read to create the pixbuf, or an error occurred in loading, then %NULL
- * will be returned.  Note that the pixbuf may not be complete until
- * @rsvg_handle_close has been called.
- *
- * Returns: the pixbuf loaded by #handle, or %NULL.
- **/
 GdkPixbuf *
-rsvg_handle_get_pixbuf (RsvgHandle *handle)
+rsvg_handle_get_pixbuf_sub (RsvgHandle *handle, const char * id)
 {
 	RsvgDimensionData dimensions;
 	GdkPixbuf *output = NULL;
@@ -88,8 +76,7 @@ rsvg_handle_get_pixbuf (RsvgHandle *handle)
 
 	cr = cairo_create (surface);
 
-	rsvg_cairo_render (cr, handle);
-
+	rsvg_cairo_render_sub (cr, handle, id);
 	rsvg_cairo_to_pixbuf(pixels, rowstride, dimensions.height);
 
 	output = gdk_pixbuf_new_from_data (pixels,
@@ -107,3 +94,22 @@ rsvg_handle_get_pixbuf (RsvgHandle *handle)
 
 	return output;
 }
+
+
+/**
+ * rsvg_handle_get_pixbuf:
+ * @handle: An #RsvgHandle
+ *
+ * Returns the pixbuf loaded by #handle.  The pixbuf returned will be reffed, so
+ * the caller of this function must assume that ref.  If insufficient data has
+ * been read to create the pixbuf, or an error occurred in loading, then %NULL
+ * will be returned.  Note that the pixbuf may not be complete until
+ * @rsvg_handle_close has been called.
+ *
+ * Returns: the pixbuf loaded by #handle, or %NULL.
+ **/
+GdkPixbuf *
+rsvg_handle_get_pixbuf (RsvgHandle *handle)
+{
+	return rsvg_handle_get_pixbuf_sub (handle, NULL);
+}
